from flask import Flask, jsonify,request
from datetime import datetime
import redis
from datetime import time
import json
import os


redis_host = 'localhost'
redis_port = 6379
r = redis.StrictRedis(host =redis_host, port= redis_port, decode_responses= True, db =0)

app = Flask(__name__)
result = {}
documents = ['ADHAR', 'UNCRD', 'PODOC', 'PANCR', 'TWIPC', 'RATCR', 'RVCER', 'VHTAX', 
             'FITCER', 'SSCER', 'LPGSV', 'CRIPC', 'INCER', 'HSCER', 'DRVLC', 'CTCER', 
             'BTCER', 'EPSER', 'PMJAY', 'ASTMT', 'CVIPC', 'EPRAN', 'SPCER', 'HLIPC', 
             'LICER', 'SKCER', 'DMCER', 'RSCER', 'HPCER', 'OSFRM', 'DTCER', 'RSBYC', 
             'SSMGR', 'ROR1B', 'HSMGR', 'IRCER', 'CDCER', 'PECER', 'PSCER', 'EGIPC', 
             'RECER', 'ITCER', 'CMCER', 'OBCER', 'NTCER', 'GOVID', 'PVCER', 'MIIPC', 
             'BHCER', 'TRIPC', 'TDCER', 'RDCER', 'SHCER', 'RMCER', 'OSCER', 'FMCER', 
             'EWCER', 'EBCER', 'CNCMY', 'SVCER', 'LWCER', 'CNDAA', 'ADCRD', 'ITICR',
             'NTMST', 'HMIPC', 'SSYCR', 'ESCER', 'NTMKS', 'DGMST', 'GPFCR', 'LHCER',
                 'CHCER', 'CPCER', 'GICER', 'LOCER', 'NTLTR', 'HVCER', 'ISMFC', 'PNCER', 
                 'SDCRD', 'EGCER', 'NNCER', 'SICRD', 'CVCER', 'SKMST', 'LCSAG', 'PRIPC', 
                 'SMCER', 'HKCER', 'DPICR', 'AGCER', 'RLCER', 'NLCER', 'WWCER', 'MNCER', 
                 'OTCER', 'DIPCR', 'LVCER', 'UNCER']


'''
#COMPLETE ROUTE PATHS LIST FOR THIS TASK:

1.http://127.0.0.1:5000/documents/adhar
2.http://127.0.0.1:5000/documents/adhar/01122000
3.http://127.0.0.1:5000/count/adhar
4.http://127.0.0.1:5000/jsonfile/data                - json file (appending Info)
5.http://127.0.0.1:5000/process_response             - json file (segregated according to date,timeframe,doctype)
6.http://127.0.0.1:5000/final_data                   - json file (Final Document)


'''



def calculate_age(dob):
    today = datetime.today()
    age = today.year - dob.year - ((today.month, today.day) < (dob.month, dob.day))
    return age
def get_age_category(age):
    if age < 10:
        return 'Under 10'
    elif age < 20:
        return '10-20'
    elif age < 30:
        return '20-30'
    elif age < 40:
        return '30-40'
    elif age < 50:
        return '40-50'
    elif age < 60:
        return '50-60'
    elif age < 70:
        return '60-70'
    elif age < 80:
        return '70-80'
    elif age < 90:
        return '80-90'
    else:
        return 'Over 90'
    
#DOC_TYPE VERIFICATION
@app.route('/documents/<string:doc_type>', methods=['GET'])
def select_document(doc_type):
    doc_type= doc_type.upper()
    if doc_type in documents :
        return f"You selected the {doc_type.upper()} document.\n Please Enter the Date of birth of format[01022015] Query parameter"
    else:
        return "Invalid document type."


#DATE_OF_BIRTH & DOC_TYPE VERFICATION
@app.route('/documents/<string:doc_type>/<string:dob>', methods=['GET'])
def get_document_by_dob(doc_type, dob):
    doc_type= doc_type.upper()
    if doc_type not in documents:
        return jsonify({'error': 'Invalid document type'})
    
    dob = datetime.strptime(dob, '%d%m%Y').date()
    age = calculate_age(dob)
    today = datetime.today().strftime('%d%m%Y')
    age_cal = get_age_category(age)
    current_time = datetime.now()
    n=current_time.strftime('%p')
    n_res=current_time.strftime('%I') 
    nxtres=f'{today}-{n_res}'
    timeframe_incr=int(n_res)+1
    timeframe=f'{str(n_res)}-{str(timeframe_incr)} {n}'
    
    result = {'doc_type': doc_type, 'age': age, 'Date': today, 'age_category': age_cal, 'Timeframe':timeframe}
    redis_key = str(result)
    
    count_key = f'{doc_type}:{datetime.utcnow().strftime("%Y-%m-%d %H")}'
    count = r.incr(count_key)
    result['count'] =count

    today=datetime.today().strftime('%d%m%y')
    current_time = datetime.now()
    n_res=current_time.strftime('%I%M%S') 
    nxtres=f'{today}-{n_res}'
    r.hset(nxtres,"Doc_type",doc_type)
    r.hset(nxtres, "Age",age)
    r.hset(nxtres,"Date",today)
    r.hset(nxtres,"Age_category", age_cal)
    r.hset(nxtres, "count",count)

        
    with open('data.json', 'r') as json_file:
        existing_data = json.load(json_file)

    if not isinstance(existing_data, list):
        existing_data = []
    existing_data.append(result)


    with open('data.json', 'w') as json_file:
        json.dump(existing_data, json_file,indent=4)
    return jsonify(result)


#GETIING INPUT JSON, ACCORDING TO DOB:
@app.route('/jsonfile/data', methods=['GET'])
def open_json_file():
    
    with open('data.json', 'r') as json_file:
        data_json= json.load(json_file)
    return jsonify(data_json)


# FOR CREATING "PROCESS.JSON", CUZ THIS DATA IS EXTRACTED & ARRANGED BASED ON TIME_FRAME, AGE_CATEGORY, DOC_TYPE

@app.route('/process_response', methods=['GET'])
def process_response():
    with open('data.json', 'r') as json_file:
        data = json.load(json_file)

    result = {
        "responses": []
    }

    for response in data:
        total_count=0
        date = response['Date']
        doc_type = response['doc_type']
        age_category = response['age_category']
        time_frame=response['Timeframe']
        age=response['age']
        count = response['count']
        if count > total_count:
                        total_count = count
        
        
        
        response_obj = next(
            (res for res in result['responses'] if res['Date'] == date),
            None
        )

        if response_obj is None:
            
            response_obj = {
                "Date": date,
                "doc_types": []
            }
            result['responses'].append(response_obj)

        

        
        doc_type_obj = next(
            (doc for doc in response_obj['doc_types'] if doc['doc_type'] == doc_type),
            None
        )
       
        if doc_type_obj is None:
           
            doc_type_obj = {
                "doc_type": doc_type,
                "time_frames": []
            }
            response_obj['doc_types'].append(doc_type_obj)

      
        time_frame_obj=next(
            (tframe for tframe in doc_type_obj['time_frames'] if tframe['Time_frame']==time_frame),
        None
        )

        if time_frame_obj is None:
            time_frame_obj={
                "Time_frame":time_frame,
                "age_categories":[]
            }

            doc_type_obj['time_frames'].append(time_frame_obj)    
        
        
        age_category_obj = next(
            (age for age in time_frame_obj['age_categories'] if age['age_category'] == age_category),
            None
        )

        if age_category_obj is None:

            age_category_obj = {
                "age_category": age_category,
                "total_count": total_count,
                "Age":age
            }
           
            time_frame_obj['age_categories'].append(age_category_obj)
    
    

    with open('process_res.json','w') as jd:
        json.dump(result,jd,indent=4)

    return jsonify(result)

#COUNT THE DOCUMENTS BASED ON THE VERIRFICATION OF DOC_TYPE
#WHICH IS A TOTAL COUNT
@app.route('/count/<string:doc_type>', methods=['GET'])
def get_count(doc_type):
    doc_type = doc_type.upper()
    total_count = 0
    
    with open('data.json','r') as read_json:
         read=json.load(read_json)
         if doc_type in documents :

            for data in read:

                if data['doc_type'].upper() == doc_type:
                    count = data['count']
                    if count > total_count:
                        total_count = count
            # answer=[{'doc_type': doc_type, 'total_count': total_count}for doc_type,total_count in read.items()]
            return jsonify({'doc_type': doc_type, 'total_count': total_count})
         else:
           return "Invalid document type."
         


#FINAL RESULT JSON; BY VERIFYING THE "PROCESS.JSON"; INCLUDING {AGE_CATEGORY,WEIGHT,DOC_TYPE}
@app.route('/final_data', methods=['GET'])
def final_data():
    with open('process_res.json', 'r') as json_file:
        data = json.load(json_file)

    result = {}

    for response in data['responses']:
        doc_types = response['doc_types']
        for doc_type_obj in doc_types:
            doc_type = doc_type_obj['doc_type']
            time_frames = doc_type_obj['time_frames']
            for time_frame_obj in time_frames:
                time_frame = time_frame_obj['Time_frame']
                age_categories = time_frame_obj['age_categories']
                for age_category_obj in age_categories:
                    age_category = age_category_obj['age_category']
                    total_count = age_category_obj['total_count']                    
                    weight = total_count / 1000
                    dict_name = f"{doc_type}_{age_category}"
                    if dict_name not in result:
                        result[dict_name] = {
                            "doc_type": doc_type,
                            "age_category": age_category,
                            "weight": 0.0
                        }

                    
                    result[dict_name]['weight'] += weight

                    r.hset(dict_name,"Doc_type", doc_type)
                    r.hset(dict_name,"age_cetegory",age_category)
                    r.hset(dict_name,"Weight",weight)

    return jsonify(result)

from main import app

if __name__ == '__main__':
    app.run(debug=True)